/* 32bitOpsIntern.h */

#ifndef __32BITOPS_INTERN

#define __32BITOPS_INTERN

/* _________ konst. _____________________ */

#define STRING_LENGTH 32
#define INSTRUCTION_LENGTH 4

#define MAX_INSTRUCTIONS STRING_LENGTH / INSTRUCTION_LENGTH 

/* __________ operation type ______________ */

#define T_LOGIC		1
#define T_ARITH		2
#define T_MOV		3
#define T_SET		4
#define T_OTHER		5
#define T_SPECIAL	6

/* ______ defines of instruction-table entries ______________________ */

/* instructions with one or more parameters have to be casted to "MicroCode" */

#define EXOR	{"EXOR", mExor, 0, T_LOGIC}
#define AND	{"AND", mAnd, 0, T_LOGIC}
#define OR	{"OR", mOr, 0, T_LOGIC}
#define NOT	{"NOT", mNot, 0, T_LOGIC}
#define ID	{"ID", mId, 0, T_LOGIC}
#define EQ	{"EQ", mEq, 0, T_LOGIC}
#define	ADC	{"ADC",mAddWithCarry , 0, T_ARITH}

#define	MOPL	{"MOPL", mMovOpPtLeft , 0, T_MOV}
#define	MOPR	{"MOPR", mMovOpPtRight , 0, T_MOV}
#define	MRPL	{"MRPL", mMovResPtLeft, 0, T_MOV}
#define	MRPR	{"MRPR", mMovResPtRight, 0, T_MOV}
#define	MPL	{"MPL", mMovPtLeft, 0, T_MOV}
#define	MPR	{"MPR", mMovPtRight, 0, T_MOV}

#define	MOV	{"MOV", mMov, 0, T_MOV}
#define	TDIR	{"TDIR", mToggleDir, 0, T_SET}
#define	TMM	{"TMM", mToggleMovMode, 0, T_SET}

#define SETP00	{"SETP00", mSetPattern00, 0, T_SET}
#define SETP01	{"SETP01", mSetPattern01, 0, T_SET}
#define SETP10	{"SETP10", mSetPattern10, 0, T_SET}
#define SETP11	{"SETP11", mSetPattern11, 0, T_SET}
#define	UNSETP	{"UNSETP", mUnSetPattern, 0, T_SET}
#define SETP	{"SETP", (MicroCode)mSetPattern, 1, T_SET}	/* following nibble is the pattern */


#define MPATL	{"MPATL", mMovToPatternLeft, 0, T_MOV}
#define MPATR	{"MPATR", mMovToPatternRight, 0, T_MOV}

#define IDON	{"IDON", mCopyOn, 0, T_SET}
#define IDOFF	{"IDOFF", mCopyOff, 0, T_SET}

#define CPON	{"CPON", mOn, 0, T_SET}
#define CPOFF	{"CPOFF", mOff, 0, T_SET}

#define SPATL	{"SPATL", mShiftToPatternLeft, 0, T_MOV}
#define SPATR	{"SPATR", mShiftToPatternRight, 0, T_MOV}

#define NOP	{"NOP", mNop, 0, T_OTHER}

#define STOP	{"STOP", mStop, 0, T_SPECIAL}
#define ERROR   {"ERROR", mError, 0, T_SPECIAL}



/* __________ decl. des MicroCodes __________________________ */


void	mExor(); 			/* logic function */
void	mAnd(); 
void	mOr();
void	mNot();
void	mId();
void	mEq();
void	mAddWithCarry();		/* not implemented yet */

void	mMovOpPtLeft();		/* one step move operations */
void	mMovOpPtRight();
void	mMovResPtLeft();
void	mMovResPtRight();
void	mMovPtLeft();		/*    both pointer */
void	mMovPtRight();	

void	mMov();
void	mMovOneStep();
void	mToggleDir();
void	mToggleMovMode();

void	mSetPattern00();
void	mSetPattern01();
void	mSetPattern10();
void	mSetPattern11();
void	mUnSetPattern();
void	mSetPattern(unsigned long int arg);		

void	mMovToPatternLeft();
void	mMovToPatternRight();

void	mCopyOn();
void	mCopyOff();

void	mOn();
void	mOff();


void	mShiftToPatternLeft();		/* onley the operand pointer */
void	mShiftToPatternRight();


void	mNop();			/* controll & special*/
void	mStop();
void	mError();

/* _______ machine io & init ____________________ */

unsigned long int	mResult();
void			mInit(unsigned long int Operand);

/* ___________ etc. functions ___________________________ */

void	printDebugInformation(int instructionNo);
void	printBin(unsigned long int data, int length);


typedef void    (*MicroCode)();
typedef void    (*MicroCode1)(unsigned long int par1);

typedef struct		/* the bin. code is defined by the position */
{
        char                    *opCode;    
        MicroCode               microCode;
	int			arrity ;   /* e.g. 1 = one 4bit arg. */
	int			type;
} InstructionTableEntry ;


typedef InstructionTableEntry InstructionTable[16] ;


#endif
